package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.model.account.Group
import java.time.LocalDate
import javax.persistence.*

@Entity
@Table(name = "recipe")
data class Recipe(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    /** The name of the recipe. It is not unique in the entire system, but is unique in the scope of a [Company]. */
    val name: String,

    val description: String,

    /** The color produced by the recipe. The string should be formatted as a hexadecimal color without the sharp (#). */
    val color: String,

    /** The gloss of the color in percents. (0-100) */
    val gloss: Byte,

    val sample: Int?,

    @Column(name = "approbation_date")
    val approbationDate: LocalDate?,

    /** A remark given by the creator of the recipe. */
    val remark: String,

    @ManyToOne
    @JoinColumn(name = "company_id")
    val company: Company,

    @OneToMany(cascade = [CascadeType.ALL], mappedBy = "recipeId")
    val mixes: List<Mix>,

    @OneToMany(cascade = [CascadeType.ALL], orphanRemoval = true)
    @JoinColumn(name = "recipe_id")
    val groupsInformation: List<RecipeGroupInformation>
) : ModelEntity

@Entity
@Table(name = "recipe_group_information")
data class RecipeGroupInformation(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    @ManyToOne
    @JoinColumn(name = "group_id")
    val group: Group,

    var note: String?,

    @OneToMany(cascade = [CascadeType.ALL], fetch = FetchType.EAGER, orphanRemoval = true)
    @JoinColumn(name = "recipe_group_information_id")
    var steps: List<RecipeStep>?
) : ModelEntity