package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.MixType
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MixTypeRepository : JpaRepository<MixType, Long> {
    /** Checks if a mix type with the given [name], [materialTypeId] and a different [id] exists. */
    @Query(
        """
        SELECT CASE WHEN(COUNT(mt.id)) > 1 THEN TRUE ELSE FALSE END
        FROM MixType mt 
        WHERE mt.name = :name AND mt.materialType.id = :materialTypeId AND mt.id <> :id
    """
    )
    fun existsByNameAndMaterialTypeAndIdNot(name: String, materialTypeId: Long, id: Long): Boolean

    /** Finds the mix type with the given [name] and [materialTypeId]. */
    @Query("SELECT m FROM MixType m WHERE m.name = :name AND m.material.materialType.id = :materialTypeId")
    fun findByNameAndMaterialType(name: String, materialTypeId: Long): MixType?

    /** Checks if a mix depends on the mix type with the given [id]. */
    @Query(
        """
        select case when(count(m.id) > 0) then false else true end
        from Mix m where m.mixType.id = :id
        """
    )
    fun isUsedByMixes(id: Long): Boolean

    /** Checks if the mix type with the given [id] is used by more than one mix. */
    @Query(
        """
        select case when(count(m.id) > 1) then false else true end
        from Mix m where m.mixType.id = :id
        """
    )
    fun isShared(id: Long): Boolean
}
