package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.logic.config.ConfigurationLogic
import dev.fyloz.colorrecipesexplorer.logic.files.WriteableFileLogic
import dev.fyloz.colorrecipesexplorer.model.ConfigurationType
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import java.net.URI

@RestController
@RequestMapping(Constants.ControllerPaths.FILE)
class FileController(
    private val fileLogic: WriteableFileLogic,
    private val configurationLogic: ConfigurationLogic
) {
    @GetMapping(produces = [MediaType.APPLICATION_OCTET_STREAM_VALUE])
    fun upload(
        @RequestParam path: String,
        @RequestParam(required = false) mediaType: String?
    ) = okFile(fileLogic.read(path), mediaType)

    @PutMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAnyAuthority('WRITE_FILE')")
    fun download(
        file: MultipartFile,
        @RequestParam path: String,
        @RequestParam(required = false) overwrite: Boolean = false
    ): ResponseEntity<Void> {
        fileLogic.write(file, path, overwrite)
        return created(path)
    }

    @DeleteMapping
    @PreAuthorize("hasAnyAuthority('WRITE_FILE')")
    fun delete(@RequestParam path: String): ResponseEntity<Void> =
        noContent {
            fileLogic.delete(path)
        }

    private fun created(path: String): ResponseEntity<Void> =
        ResponseEntity
            .created(URI.create("${configurationLogic.get(ConfigurationType.INSTANCE_URL)}${Constants.ControllerPaths.FILE}?path=$path"))
            .build()
}
