package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.dtos.TouchUpKitDto
import dev.fyloz.colorrecipesexplorer.logic.TouchUpKitLogic
import org.springframework.context.annotation.Profile
import org.springframework.core.io.Resource
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.TOUCH_UP_KIT)
@Profile("!emergency")
@PreAuthorize("hasAuthority('VIEW_TOUCH_UP_KITS')")
class TouchUpKitController(
    private val touchUpKitLogic: TouchUpKitLogic
) {
    @GetMapping
    fun getAll() = ok(touchUpKitLogic.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) = ok(touchUpKitLogic.getById(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun save(@Valid @RequestBody touchUpKit: TouchUpKitDto) =
        created<TouchUpKitDto>(Constants.ControllerPaths.TOUCH_UP_KIT) {
            touchUpKitLogic.save(touchUpKit)
        }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun update(@Valid @RequestBody touchUpKit: TouchUpKitDto) = noContent {
        touchUpKitLogic.update(touchUpKit)
    }

    @PutMapping("{id}/complete")
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun complete(@PathVariable id: Long) = noContent {
        touchUpKitLogic.complete(id)
    }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun deleteById(@PathVariable id: Long) = noContent {
        touchUpKitLogic.deleteById(id)
    }

    @GetMapping("pdf")
    fun getJobPdf(@RequestParam project: String): ResponseEntity<Resource> {
        with(touchUpKitLogic.generateJobPdfResource(project)) {
            return ResponseEntity.ok()
                .header("Content-Disposition", "filename=TouchUpKit_$project.pdf")
                .contentLength(this.contentLength())
                .contentType(MediaType.APPLICATION_PDF)
                .body(this)
        }
    }
}
