package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.MixMixTypeDto
import dev.fyloz.colorrecipesexplorer.model.MixMixType
import dev.fyloz.colorrecipesexplorer.repository.MixMixTypeRepository

interface MixMixTypeService : Service<MixMixTypeDto, MixMixType, MixMixTypeRepository> {
    fun getAllByMixId(mixId: Long): List<MixMixTypeDto>

    fun saveAllForMixId(mixMixTypes: List<MixMixTypeDto>, mixId: Long): List<MixMixTypeDto>
}

@ServiceComponent
class DefaultMixMixTypeService(repository: MixMixTypeRepository, private val mixTypeService: MixTypeService) :
    BaseService<MixMixTypeDto, MixMixType, MixMixTypeRepository>(repository), MixMixTypeService {
    override fun getAllByMixId(mixId: Long) = repository.findAllByMixId(mixId).map(::toDto)

    override fun saveAllForMixId(mixMixTypes: List<MixMixTypeDto>, mixId: Long): List<MixMixTypeDto> {
        repository.deleteAllByMixId(mixId)

        mixMixTypes.forEach { saveForMixId(it, mixId) }
        return getAllByMixId(mixId)
    }

    fun saveForMixId(mixMixType: MixMixTypeDto, mixId: Long) =
        repository.saveForMixId(mixMixType.id, mixMixType.mixType.id, mixId, mixMixType.quantity, mixMixType.position)

    override fun toDto(entity: MixMixType) =
        MixMixTypeDto(entity.id, mixTypeService.toDto(entity.mixType), entity.quantity, entity.position)

    override fun toEntity(dto: MixMixTypeDto) =
        MixMixType(dto.id, mixTypeService.toEntity(dto.mixType), dto.quantity, dto.position)
}