package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.TouchUpKitDto
import dev.fyloz.colorrecipesexplorer.dtos.TouchUpKitProductDto
import dev.fyloz.colorrecipesexplorer.logic.config.ConfigurationLogic
import dev.fyloz.colorrecipesexplorer.model.ConfigurationType
import dev.fyloz.colorrecipesexplorer.model.touchupkit.TouchUpKit
import dev.fyloz.colorrecipesexplorer.model.touchupkit.TouchUpKitProduct
import dev.fyloz.colorrecipesexplorer.repository.TouchUpKitRepository
import java.time.LocalDate
import java.time.Period

interface TouchUpKitService : Service<TouchUpKitDto, TouchUpKit, TouchUpKitRepository> {
    /** Updates the [completionDate] of the touch up kit with the given [id]. */
    fun updateCompletionDateById(id: Long, completionDate: LocalDate)
}

@ServiceComponent
class DefaultTouchUpKitService(repository: TouchUpKitRepository, private val configurationLogic: ConfigurationLogic) :
    BaseService<TouchUpKitDto, TouchUpKit, TouchUpKitRepository>(repository), TouchUpKitService {
    override fun updateCompletionDateById(id: Long, completionDate: LocalDate) =
        repository.updateCompletionDateById(id, completionDate)

    override fun toDto(entity: TouchUpKit) =
        TouchUpKitDto(
            entity.id!!,
            entity.project,
            entity.buggy,
            entity.company,
            entity.quantity,
            entity.shippingDate,
            entity.completionDate,
            entity.completionDate != null,
            isExpired(entity),
            entity.finish.split(LIST_DELIMITER),
            entity.material.split(LIST_DELIMITER),
            entity.content.map(::touchUpKitProductToDto)
        )

    private fun touchUpKitProductToDto(entity: TouchUpKitProduct) =
        TouchUpKitProductDto(entity.id!!, entity.name, entity.description, entity.quantity, entity.ready)

    override fun toEntity(dto: TouchUpKitDto) =
        TouchUpKit(
            dto.id,
            dto.project,
            dto.buggy,
            dto.company,
            dto.quantity,
            dto.shippingDate,
            dto.completionDate,
            dto.finish.joinToString(LIST_DELIMITER),
            dto.material.joinToString(LIST_DELIMITER),
            dto.content.map(::touchUpKitProductToEntity)
        )

    private fun touchUpKitProductToEntity(dto: TouchUpKitProductDto) =
        TouchUpKitProduct(dto.id, dto.name, dto.description, dto.quantity, dto.ready)

    private fun isExpired(touchUpKit: TouchUpKit) =
        with(Period.parse(configurationLogic.getContent(ConfigurationType.TOUCH_UP_KIT_EXPIRATION))) {
            touchUpKit.completionDate != null && touchUpKit.completionDate.plus(this) < LocalDate.now()
        }

    companion object {
        private const val LIST_DELIMITER = ";"
    }
}